/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef CONSUMER_H
#define CONSUMER_H

#include "rgma/ResourceImpl.h"
#include "rgma/ResourceEndpointList.h"
#include "rgma/ResourceEndpoint.h"
#include "rgma/ResultSet.h"
#include "rgma/TimeInterval.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RemoteException.h"
#include "rgma/StringList.h"

namespace glite
{
    namespace rgma
    {
        /**
         * A client uses a Consumer to retrieve data from one or more producers.  A
         * Consumer is created using a ConsumerFactory.
         *
         * This is an abstract class which should be extended to produce a consumer
         *
         * @see ConsumerFactory#createConsumer
         */
        class Consumer: virtual public ResourceImpl
        {

            protected:
                Consumer(){}
                bool m_isExecuting;
                ResourceEndpointList m_relevantProducers;
                StringList m_voNames;
            public:

                virtual ~Consumer(){}
                /**
                 * Determines whether the last query is still executing on the server.
                 * @deprecated Use ResultSet#endOfResults
                 * @return True if the query is still executing.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 */
                virtual bool isExecuting() throw(UnknownResourceException, RemoteException, RGMAException) = 0;

                /**
                 * Aborts the current query.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 */
                virtual void abort() throw(UnknownResourceException, RemoteException) = 0;
                /**
                 * Returns the number of tuples that are available to pop.
                 * @deprecated
                 * @return The number of tuples that are available to pop.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 */
                virtual int count() throw(UnknownResourceException, RemoteException) = 0;
                /**
                 * Determines if the last query has aborted.
                 *
                 * @return True if the query was aborted and didn't stop of its own accord.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 */
                virtual bool hasAborted()throw(UnknownResourceException, RemoteException) = 0;
                /**
                 * Retrieves at most <code>maxCount</code> tuples from the consumer that it
                 * has received from producers.
                 *
                 * @param maxCount The maximum number of tuples to retrieve.
                 * @param results a resultSet object to be populated with the results of the pop action
                 *
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the query hasn't been started.
                 */
                virtual void pop(ResultSet& results, int maxCount)throw(UnknownResourceException, RemoteException, RGMAException) = 0;
                /**
                 * Retrieves all data from the consumer that it has received from
                 * producers.
                 *
                 * @param results a resultSet object to be populated with the results of the popAll action
                 * @deprecated use pop(ResultSet& results, int maxCount)
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the query hasn't been started.
                 */
                virtual void popAll(ResultSet& results)throw(UnknownResourceException, RemoteException, RGMAException) = 0;
                /**
                 * Starts this consumer's query, using the mediator to locate producers,
                 * terminating after the specified time interval.  There is no infinite
                 * interval; just set the timeout to be very large if this is the effect
                 * you want (e.g. 1000 days).
                 *
                 * @param timeout Time interval after which the query will automatically be
                 *        aborted.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the query is currently executing.
                 * @deprecated
                 */
                virtual void start(TimeInterval timeout)throw(UnknownResourceException, RemoteException, RGMAException) = 0;
                /**
                 * Starts this consumer's query, contacting the specified producers,
                 * terminating after the specified time interval. There is no infinite
                 * interval; just set the timeout to be very large if this is the effect
                 * you want (e.g. 1000 days).
                 *
                 * @param timeout Time interval after which the query will automatically be
                 *        aborted.
                 * @param producers List of producers to contact.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the query is currently executing.
                 */
                virtual void start(TimeInterval timeout, ResourceEndpointList producers)
                    throw(UnknownResourceException, RemoteException, RGMAException) = 0;

                /**
                 * Gets the version of this implementation.
                 *
                 * @return The version number.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 */
                static std::string getVersion()throw(RemoteException);

                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param parameter Parameter to property.
                 *
                 * @return Current value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name, std::string parameter)
                    throw(RemoteException, RGMAException);
                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 * @param parameter Parameter to property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value, std::string parameter)
                    throw(RemoteException, RGMAException);

                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 *
                 * @return Current value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name)
                    throw(RemoteException, RGMAException);
                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value)
                    throw(RemoteException, RGMAException);

        };
    }
}
#endif                                            // CONSUMER_H
