/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef CONSUMERIMPL_H
#define CONSUMERIMPL_H

#include "rgma/Consumer.h"
#include "rgma/ResourceEndpoint.h"
#include "rgma/ResourceEndpointList.h"
#include "rgma/TimeInterval.h"
#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RGMAException.h"
#include "rgma/Resource.h"
#include "rgma/StringList.h"
#include "rgma/ProducerProperties.h"
#include "rgma/QueryProperties.h"
#include "rgma/Units.h"
#include "rgma/ResultSet.h"
#include "rgma/ResultSetMetaData.h"
#include "rgma/Tuple.h"
#include "rgma/Types.h"

namespace glite
{
    namespace rgma
    {
        /**
         * Implementation of a consumer
         */
        class ConsumerImpl : virtual public Consumer
        {

            public:

                ConsumerImpl(TimeInterval terminationInterval,
                    std::string selectStatement, QueryProperties queryProperties)throw(RemoteException, RGMAException);

                ConsumerImpl(ResourceEndpoint& resourceEndpoint)throw(RemoteException, RGMAException);

                virtual ~ConsumerImpl();
                /**
                 * Determines whether the last query is still executing.
                 * @deprecated Use ResultSet#endOfResults
                 * @return True if the query is still executing.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 */
                bool isExecuting() throw(UnknownResourceException, RemoteException, RGMAException);

                /**
                 * Aborts the current query.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 */
                void abort() throw(UnknownResourceException, RemoteException);
                /**
                 * Returns the number of tuples that are available to pop.
                 * @deprecated
                 * @return The number of tuples that are available to pop.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 */
                int count() throw(UnknownResourceException, RemoteException);
                /**
                 * Determines if the last query has aborted.
                 *
                 * @return True if the query was aborted and didn't stop of its own accord.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 */
                bool hasAborted()throw(UnknownResourceException, RemoteException);
                /**
                 * Retrieves at most <code>maxCount</code> tuples from the consumer that it
                 * has received from producers.
                 *
                 * @param maxCount The maximum number of tuples to retrieve.
                 * @param results a resultSet object to be populated with the results of the pop action
                 *
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the query hasn't been started.
                 */
                void pop(ResultSet& results, int maxCount)throw(UnknownResourceException, RemoteException, RGMAException);
                /**
                 * Retrieves all data from the consumer that it has received from
                 * producers.
                 * @deprecated
                 * @param results a resultSet object to be populated with the results of the popAll action
                 *
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the query hasn't been started.
                 */
                void popAll(ResultSet& results)throw(UnknownResourceException, RemoteException, RGMAException);
                /**
                 * Starts this consumer's query, using the mediator to locate producers,
                 * terminating after the specified time interval.  There is no infinite
                 * interval; just set the timeout to be very large if this is the effect
                 * you want (e.g. 1000 days).
                 *
                 * @param timeout Time interval after which the query will automatically be
                 *        aborted.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the query is currently executing.
                 *
                 */
                void start(TimeInterval timeout)throw(UnknownResourceException, RemoteException, RGMAException);
                /**
                 * Starts this consumer's query, contacting the specified producers,
                 * terminating after the specified time interval. There is no infinite
                 * interval; just set the timeout to be very large if this is the effect
                 * you want (e.g. 1000 days).
                 *
                 * @param timeout Time interval after which the query will automatically be
                 *        aborted.
                 * @param producers List of producers to contact.
                 *
                 * @throws UnknownResourceException If the consumer resource is not found.
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the query is currently executing.
                 */
                void start(TimeInterval timeout, ResourceEndpointList producers)
                    throw(UnknownResourceException, RemoteException, RGMAException);

        };
    }
}
#endif                                            // CONSUMERWRAPPER_H
