/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef ONDEMANDPRODUCER_H
#define ONDEMANDPRODUCER_H

#include "rgma/ResourceImpl.h"
#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RGMAException.h"
#include <string>

namespace glite
{
    namespace rgma
    {
        /** A client uses an OnDemandProducer to publish data into R-GMA when the cost
         * of creating each message is high. The OnDemandProducer only generates
         * messages when there is a specific query from a Consumer.
         */
        class OnDemandProducer: public ResourceImpl
        {
            public:

                virtual ~OnDemandProducer(){}
                /**
                 * Declares a static table into which this Producer can publish.  A subset
                 * of a table can be declared using a predicate.  A static table has no
                 * MeasurementDate/Time associated with each tuple.
                 *
                 * @param tableName The name of the table to publish into.
                 * @param predicate An SQL WHERE clause defining the subset of a table that
                 *        this Producer will publish.  To publish to the whole table, an
                 *        empty predicate can be defined using "".
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If table <code>tableName</code> is not in the
                 *         Schema. If the predicate is invalid.
                 */
                virtual void declareStaticTable(std::string tableName, std::string predicate)
                    throw(RemoteException, RGMAException, UnknownResourceException) = 0;
                /**
                 * Gets the version of this implementation.
                 *
                 * @return The version number.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 */
                static std::string getVersion()throw(RemoteException);

                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param parameter Parameter to property
                 *
                 * @return Current value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name, std::string parameter)
                    throw(RemoteException, RGMAException);
                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 *
                 * @return Current value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name)
                    throw(RemoteException, RGMAException);
                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 *  @param parameter Parameter to property
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value, std::string parameter)
                    throw(RemoteException, RGMAException);

                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value)
                    throw(RemoteException, RGMAException);

        };
    }
}
#endif                                            // ONDEMANDPRODUCER_H
