/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef PRIMARYPRODUCER_H
#define PRIMARYPRODUCER_H
#include "rgma/ResourceEndpoint.h"
#include "rgma/TimeInterval.h"
#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RGMAException.h"
#include "rgma/ResourceImpl.h"
#include "rgma/StringList.h"
#include "rgma/TupleStoreList.h"

#include <string>
namespace glite
{
    namespace rgma
    {
        /**
         * A client uses a PrimaryProducer to publish information into R-GMA.
         */
        class PrimaryProducer: public ResourceImpl
        {
            private:

            public:

                virtual ~PrimaryProducer(){}

                /**
                 * Gets the retention period that is applied to history tuples for the
                 * specified table.
                 *
                 * @param tableName Name of the table
                 *
                 * @return The retention period for history tuples.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found.
                 * @throws RGMAException If the table has not been declared.
                 */
                virtual TimeInterval getHistoryRetentionPeriod(std::string tableName)
                    = 0;
                /**
                 * Gets the retention period that is applied to latest tuples for the
                 * specified table.
                 *
                 * @param tableName Name of the table
                 *
                 * @return The retention period for latest tuples.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found.
                 * @throws RGMAException If the table has not been declared.
                 */
                virtual TimeInterval getLatestRetentionPeriod(std::string tableName)
                    throw(RemoteException, UnknownResourceException, RGMAException) = 0;
                /**
                 * Declares a table, specifying the retention period for history and latest
                 * tuples.  The latestRetentionPeriod specifies the time for which a
                 * latest tuple is valid.  After this time, the tuple is removed.  Tuples
                 * will be removed from storage when the history retention period expires.
                 *
                 * @param tableName The name of the table to declare.
                 * @param predicate An SQL WHERE clause defining the subset of a table that
                 *        this Producer will publish.  To publish to the whole table, an
                 *        empty predicate can be used.
                 * @param historyRetentionPeriod The retention period for history tuples
                 * @param latestRetentionPeriod The retention period for latest tuples
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the tableName is unknown. If the predicate is
                 *         invalid. If either xxxRetentionPeriod is invalid.
                 */
                virtual void declareTable(std::string tableName, std::string predicate,
                    TimeInterval historyRetentionPeriod,
                    TimeInterval latestRetentionPeriod)
                    throw(RemoteException, UnknownResourceException, RGMAException) = 0;
                /**
                 * Publishes data by inserting a tuple into a table, both specified by the
                 * SQL INSERT statement.
                 *
                 * @param insertStatement An SQL INSERT statement providing the data to
                 *        publish and the table into which to put it.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the table in the insert has not been declared.
                 *         If the inserted tuple does not match its table's declared
                 *         predicate
                 *
                 * @see #insertList
                 */
                virtual void insert(std::string insertStatement)
                    throw(RemoteException, UnknownResourceException, RGMAException) = 0;
                /**
                 * Publishes data by inserting a tuple into a table, both specified by the
                 * SQL INSERT statement.
                 *
                 * @param insertStatement An SQL INSERT statement providing the data to
                 *        publish and the table into which to put it.
                 * @param latestRetentionPeriod Latest retention period for this tuple (overrides
                 *        LRP defined for table).
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the table in the insert has not been declared.
                 *         If the inserted tuple does not match its table's declared
                 *         predicate
                 *
                 * @see #insertList
                 * @see #insert
                 */
                virtual void insert(std::string insertStatement, TimeInterval latestRetentionPeriod)
                    throw(RemoteException, UnknownResourceException, RGMAException) = 0;

                /**
                 * Publishes using a list of SQL INSERT statements.
                 *
                 * @param insertStatements A list of SQL INSERT statements.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the table in any insert has not been declared.
                 *         If any inserted tuple does not match its table's declared
                 *         predicate
                 *
                 * @see #insert
                 */
                virtual void insertList(StringList insertStatements)
                    throw(RemoteException, UnknownResourceException, RGMAException) = 0;

                /**
                 * Publishes using a list of SQL INSERT statements.
                 *
                 * @param insertStatements A list of SQL INSERT statements.
                 * @param latestRetentionPeriod the retention period for latest queries
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the table in any insert has not been declared.
                 *         If any inserted tuple does not match its table's declared
                 *         predicate
                 *
                 * @see #insert
                 */
                virtual void insertList(StringList insertStatements, TimeInterval latestRetentionPeriod)
                    throw(RemoteException, UnknownResourceException, RGMAException) = 0;

                /**
                 * Gets the version of this implementation.
                 *
                 * @return The version number.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 */
                static std::string getVersion()throw(RemoteException);

                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param parameter Parameter to property
                 *
                 * @return Current value of property (can be <code>NULL</code>).
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name, std::string parameter)
                    throw(RemoteException, RGMAException);
                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 *
                 * @return Current value of property (can be <code>NULL</code>).
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name)
                    throw(RemoteException, RGMAException);
                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value)
                    throw(RemoteException, RGMAException);

                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 * @param parameter Parameter to property
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value, std::string parameter)
                    throw(RemoteException, RGMAException);

                /**
                 * Returns a list of existing tuple stores that can be used by
                 * Primary and Secondary Producers.
                 *
                 * @return A TupleStoreList object.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If no endpointReference is defined in properties.
                 */
                static TupleStoreList listTupleStores() throw( RemoteException, RGMAException);

                /**
                 * Permanently deletes a named tuple store.
                 *
                 * @param logicalName Name of tuple store.
                 *
                 * @throws RGMAException If the store does not exist or if it is in use by a producer.  If no
                 * endpointReference is defined in properties.
                 * @throws RemoteException If the service could not be contacted.
                 */
                static void dropTupleStore(std::string logicalName) throw (RGMAException, RemoteException);

        };
    }
}
#endif                                            // PRIMARYPRODUCER_H
