#ifndef PRIMARYPRODUCERIMPL_H
#define PRIMARYPRODUCERIMPL_H
#include "rgma/ResourceEndpoint.h"
#include "rgma/TimeInterval.h"
#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RGMAException.h"
#include "rgma/Resource.h"
#include "rgma/StringList.h"
#include "rgma/PrimaryProducer.h"
#include "rgma/ProducerProperties.h"
#include "rgma/Properties.h"

#include <string>
namespace glite
{
    namespace rgma
    {
        /**
         * Implementation of a producer
         */
        class PrimaryProducerImpl : public PrimaryProducer
        {
            private:
                void createProducer(TimeInterval terminationInterval,
                    ProducerProperties producerProperties) throw( RGMAException , RemoteException );

            public:
                /**
                 * @deprecated PrimaryProducerImpl(ResourceEndpoint& endPoint)
                                 throw(RemoteException, RGMAException);
                 * */
                PrimaryProducerImpl(TimeInterval terminationInterval,
                    ProducerProperties producerProperties,
                    StringList voNames) throw( RGMAException);

                PrimaryProducerImpl(TimeInterval terminationInterval,
                    ProducerProperties producerProperties) throw( RGMAException, RemoteException);

                PrimaryProducerImpl(ResourceEndpoint& endPoint)
                    throw(RemoteException, RGMAException, RemoteException);

                virtual ~PrimaryProducerImpl(){}

                /**
                 * Gets the retention period that is applied to history tuples for the
                 * specified table.
                 *
                 * @param tableName Name of the table
                 *
                 * @return The retention period for history tuples.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found.
                 * @throws RGMAException If the table has not been declared.
                 */
                TimeInterval getHistoryRetentionPeriod(std::string tableName)
                 throw(RemoteException, UnknownResourceException, RGMAException);
                /**
                 * Gets the retention period that is applied to latest tuples for the
                 * specified table.
                 *
                 * @param tableName Name of the table
                 *
                 * @return The retention period for latest tuples.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found.
                 * @throws RGMAException If the table has not been declared.
                 */
                TimeInterval getLatestRetentionPeriod(std::string tableName)
                    throw(RemoteException, UnknownResourceException, RGMAException);
                /**
                 * Declares a table, specifying the retention period for history and latest
                 * tuples.  The latestRetentionPeriod specifies the time for which a
                 * latest tuple is valid.  After this time, the tuple is removed.  Tuples
                 * will be removed from storage when the history retention period expires.
                 *
                 * @param tableName The name of the table to declare.
                 * @param predicate An SQL WHERE clause defining the subset of a table that
                 *        this Producer will publish.  To publish to the whole table, an
                 *        empty predicate can be used.
                 * @param historyRetentionPeriod The retention period for history tuples
                 * @param latestRetentionPeriod The retention period for latest tuples
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the tableName is unknown. If the predicate is
                 *         invalid. If either xxxRetentionPeriod is invalid.
                 */
                void declareTable(std::string tableName, std::string predicate,
                    TimeInterval historyRetentionPeriod,
                    TimeInterval latestRetentionPeriod)
                    throw(RemoteException, UnknownResourceException, RGMAException);
                /**
                 * Publishes data by inserting a tuple into a table, both specified by the
                 * SQL INSERT statement.
                 *
                 * @param insertStatement An SQL INSERT statement providing the data to
                 *        publish and the table into which to put it.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the table in the insert has not been declared.
                 *         If the inserted tuple does not match its table's declared
                 *         predicate
                 *
                 * @see #insertList
                 */
                void insert(std::string insertStatement)
                    throw(RemoteException, UnknownResourceException, RGMAException);
                /**
                 * Publishes data by inserting a tuple into a table, both specified by the
                 * SQL INSERT statement.
                 *
                 * @param insertStatement An SQL INSERT statement providing the data to
                 *        publish and the table into which to put it.
                 * @param latestRetentionPeriod Latest retention period for this tuple (overrides
                 *        LRP defined for table).
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the table in the insert has not been declared.
                 *         If the inserted tuple does not match its table's declared
                 *         predicate
                 *
                 * @see #insertList
                 * @see #insert
                 */
                void insert(std::string insertStatement, TimeInterval latestRetentionPeriod)
                    throw(RemoteException, UnknownResourceException, RGMAException);

                /**
                 * Publishes using a list of SQL INSERT statements.
                 *
                 * @param insertStatements A list of SQL INSERT statements.
                 * @param latestRetentionPeriod the retention period for latest queries
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the table in any insert has not been declared.
                 *         If any inserted tuple does not match its table's declared
                 *         predicate
                 *
                 * @see #insert
                 */
                void insertList(StringList insertStatements)
                    throw(RemoteException, UnknownResourceException, RGMAException);

                /**
                 * Publishes using a list of SQL INSERT statements.
                 *
                 * @param insertStatements A list of SQL INSERT statements.
                 * @param latestRetentionPeriod the retention period for latest queries
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the producer resource could not be
                 *         found
                 * @throws RGMAException If the table in any insert has not been declared.
                 *         If any inserted tuple does not match its table's declared
                 *         predicate
                 *
                 * @see #insert
                 */
                void PrimaryProducerImpl::insertList(StringList insertStatements, TimeInterval latestRetentionPeriod)
                    throw(RemoteException, UnknownResourceException, RGMAException);

        };
    }
}
#endif
