/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef PRODUCERFACTORY_H
#define PRODUCERFACTORY_H

#include "rgma/TimeInterval.h"
#include "rgma/URI.h"
#include "rgma/StringList.h"
#include "rgma/ProducerProperties.h"
#include "rgma/OnDemandProducer.h"
#include "rgma/SecondaryProducer.h"
#include "rgma/PrimaryProducer.h"
#include "rgma/Resource.h"
#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/ResourceEndpoint.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/TupleStoreList.h"

namespace glite
{
    namespace rgma
    {
        /**
         * A factory for Primary, Secondary and OnDemand Producers.
         */
        class ProducerFactory
        {
            public:

                virtual ~ProducerFactory(){}
                /**
                 * Creates an on-demand producer.
                 *
                 * @param terminationInterval The interval after which this Producer will
                 *        be removed from the remote service if there has been no contact
                 *        from the API.
                 * @param url The URL for the system that will respond to queries.
                 * @param voNames List of VO names to publish to.
                 *
                 * @return A new on-demand producer.
                 * @deprecated use OnDemandProducer* createOnDemandProducer(
                 *                          TimeInterval terminationInterval,
                 *                          URI uri)
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If no endpointReference is defined in properties.
                 *         If the URI is invalid.
                 */
                virtual OnDemandProducer* createOnDemandProducer(
                    TimeInterval terminationInterval,
                    URI uri, StringList voNames)throw(RemoteException, RGMAException) = 0;
                /**
                 * Creates an on-demand producer.
                 *
                 * @param terminationInterval The interval after which this Producer will
                 *        be removed from the remote service if there has been no contact
                 *        from the API.
                 * @param url The URL for the system that will respond to queries.
                 *
                 * @return A new on-demand producer.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If no endpointReference is defined in properties.
                 *         If the URI is invalid.
                 */
                virtual OnDemandProducer* createOnDemandProducer(
                    TimeInterval terminationInterval,
                    URI uri)
                    throw (RemoteException, RGMAException) = 0;
                /**
                 * Reconnects to an existing OnDemandProducer and checks that it is still active
                 * by sending a showSignOfLife call.
                 *
                 * @param resourceEndpoint Endpoint of the OnDemandProducer.
                 *
                 * @return A reference to the OnDemandProducer.
                 */
                virtual OnDemandProducer* reconnectOnDemandProducer(ResourceEndpoint resourceEndpoint)
                    throw (RGMAException, RemoteException, UnknownResourceException) = 0;

                /**
                 * Creates a primary producer that uses the specified data storage and
                 * producer type.
                 *
                 * @param terminationInterval The interval after which this Producer will
                 *        be removed from the remote service if there has been no contact
                 *        from the API.
                 * @param producerProperties a ProducerProperties object
                 * @param voNames List of VO names to publish to.
                 *
                 * @return A new primary producer.
                 * @deprecated use createPrimaryProducer(
                    TimeInterval terminationInterval, ProducerProperties producerProperties)
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If no endpointReference is defined in properties.
                 *         If the storage location is invalid.
                 */
                virtual PrimaryProducer* createPrimaryProducer(
                    TimeInterval terminationInterval,
                    ProducerProperties producerProperties,
                    StringList voNames)throw(RemoteException, RGMAException) = 0;
                /**
                 * Creates a primary producer that uses the specified data storage and
                 * producer type.
                 *
                 * @param terminationInterval The interval after which this Producer will
                 *        be removed from the remote service if there has been no contact
                 *        from the API.
                 * @param producerProperties a ProducerProperties object
                 *
                 * @return A new primary producer.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If no endpointReference is defined in properties.
                 *         If the storage location is invalid.
                 */
                virtual PrimaryProducer* createPrimaryProducer(
                    TimeInterval terminationInterval, ProducerProperties producerProperties)
                    throw (RemoteException, RGMAException) = 0;
                /**
                 * Reconnects to an existing PrimaryProducer and checks that it is still active
                 * by sending a showSignOfLife call.
                 *
                 * @param resourceEndpoint Endpoint of the PrimaryProducer.
                 *
                 * @return A reference to the PrimaryProducer.
                 */
                virtual PrimaryProducer* reconnectPrimaryProducer(ResourceEndpoint resourceEndpoint)
                    throw (RGMAException, RemoteException, UnknownResourceException)= 0;

                /**
                 * Creates a secondary producer, that uses a primary producer with the
                 * specified attributes to republish information.
                 *
                 * @param terminationInterval The interval after which this Producer will
                 *        be removed from the remote service if there has been no contact
                 *        from the API.
                 * @param producerProperties a ProducerProperties object
                 * @param voNames List of VO names to publish to.
                 *
                 * @return A new secondary producer.
                 * @deprecated use createSecondaryProducer(
                 *           TimeInterval terminationInterval,
                 *          ProducerProperties producerProperties)
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If no endpointReference is defined in properties.
                 *         If the storage location is invalid.
                 */
                virtual  SecondaryProducer* createSecondaryProducer(
                    TimeInterval terminationInterval,
                    ProducerProperties producerProperties,
                    StringList voNames)throw(RemoteException, RGMAException) = 0;
                /**
                 * Creates a secondary producer, that uses a primary producer with the
                 * specified attributes to republish information.
                 *
                 * @param terminationInterval The interval after which this Producer will
                 *        be removed from the remote service if there has been no contact
                 *        from the API.
                 * @param producerProperties a ProducerProperties object
                 *
                 * @return A new secondary producer.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If no endpointReference is defined in properties.
                 *         If the storage location is invalid.
                 */
                virtual  SecondaryProducer* createSecondaryProducer(
                    TimeInterval terminationInterval,
                    ProducerProperties producerProperties)
                    throw (RemoteException, RGMAException) = 0;

                /**
                 * Reconnects to an existing SecondaryProducer and checks that it is still active
                 * by sending a showSignOfLife call.
                 *
                 * @param resourceEndpoint Endpoint of the SecondaryProducer.
                 *
                 * @return A reference to the SecondaryProducer.
                 */
                virtual SecondaryProducer* reconnectSecondaryProducer(ResourceEndpoint resourceEndpoint)
                    throw (RGMAException, RemoteException, UnknownResourceException)= 0;

        };
    }
}
#endif                                            // PRODUCERFACTORY_H
