/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef PRODUCERPROPERTIES_H
#define PRODUCERPROPERTIES_H

#include "rgma/Storage.h"
#include "rgma/RGMAException.h"

namespace glite
{
    namespace rgma
    {
        /**
         * Properties of a Primary or Secondary Producer.
         */
        class ProducerProperties
        {
            private:

                bool m_history;
                bool m_latest;
                bool m_signalSlowConsumer;
                Storage m_storage;

            protected:

            public:
                /** Producer supports HISTORY queries. */
                const static int HISTORY;

                /** Producer supports LATEST queries. */
                const static int LATEST;

                /** Producer supports CONTINUOUS queries. */
                const static int CONTINUOUS;

                /**
                 * Creates a new ProducerProperties object.
                 *
                 * @param storage The storage location for tuples.
                 * @param supportedQueries Queries supported by producer (All query types support CONTINUOUS queries). Possible values are:
                 *        <code>0</code>, <code>HISTORY</code>, <code>LATEST</code> or <code>HISTORY+LATEST</code>.
                 * @param ignoreSlowConsumers Set <code>true</code> if the Producer should ignore slow Consumers if they are
                 *        preventing tuple deletion.
                 *
                 * @see #ProducerProperties(Storage, int)
                 * @deprecated use ProducerProperties(Storage, int)
                 */
                ProducerProperties(const Storage storage, int supportedQueries, bool ignoreSlowConsumers);

                /**
                 * Creates a new ProducerProperties object.
                 *
                 * @param storage The storage location for tuples.
                 * @param supportedQueries Queries supported by producer (All query types support CONTINUOUS queries). Possible values are:
                 *        <code>0</code>, <code>HISTORY</code>, <code>LATEST</code> or <code>HISTORY+LATEST</code>.
                 *
                 * @throws RGMAException if supportedQuery is not supported
                 *
                 *
                 */
                ProducerProperties(const Storage storage = Storage::MEMORY, int supportedQueries = 0)throw(RGMAException);

                virtual ~ProducerProperties();

                ProducerProperties( const ProducerProperties& producerProperties);

                ProducerProperties& operator=(const ProducerProperties& producerProperties);

                /**
                 * Determines the Producer's support for history queries.
                 *
                 * @return <code>true</code> if the Producer supports history queries.
                 */

                bool isHistory();

                /**
                 * Determines the Producer's support for latest queries.
                 *
                 * @return <code>true</code> if the Producer supports latest queries.
                 */
                bool isLatest();

                /**
                 * Determines if the Producer will block on slow Consumers.
                 *
                 * @return <code>true</code> if the Producer blocks on slow Consumers.
                 */
                bool isSignalSlowConsumer();

                /**
                 * Gets the storage location for this Producer.
                 *
                 * @return The Storage object for this Producer.
                 */
                const Storage getStorage();

        };
    }
}
#endif                                            // PRODUCERPROPERTIES_H
