/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef PRODUCERTYPE_H
#define PRODUCERTYPE_H
#include <string>

namespace glite
{
    namespace rgma
    {
        /**
         * Attributes of a Producer.
         */
        class ProducerType
        {
            public:
                /**
                 * Creates a new ProducerType instance.
                 *
                 * @param history Support HISTORY queries.
                 * @param latest Support LATEST queries.
                 * @param continuous Support CONTINUOUS queries.
                 * @param isStatic Support STATIC queries.
                 * @param republisher Is part of a republisher.
                 */
                ProducerType(bool history = false, bool latest = false, bool continuous = true, bool isStatic = false, bool republisher = false);
                /**
                 * Creates a new ProducerType instance.
                 *
                 * @param history Support HISTORY queries.
                 * @param latest Support LATEST queries.
                 * @param continuous Support CONTINUOUS queries.
                 * @param isStatic Support STATIC queries.
                 * @param republisher Is part of a republisher.
                 * @param ignoreSlowConsumers DOCUMENT ME!
                 * @deprecated
                 */
                ProducerType(bool history, bool latest, bool continuous, bool isStatic, bool republisher, bool ignoreSlowConsumers);
                ~ProducerType();
                ProducerType( const ProducerType& type );
                ProducerType& operator=(const ProducerType& type );
                bool operator==(const ProducerType& type );

                /**
                 * Is the producer a secondary producer?
                 *
                 * @return Returns secondaryProducer.
                 */
                bool isSecondaryProducer() ;

                /**
                 * Is the producer a secondary producer?
                 *
                 * @return Returns secondaryProducer.
                 *
                 * @deprecated @see #isSecondaryProducer()
                 */
                bool isRepublisher();

                /**
                 * Does the producer support continuous queries?
                 *
                 * @return Returns the continuous.
                 */
                bool isContinuous();

                /**
                 * Does the producer support static queries?
                 *
                 * @return Returns the static.
                 */
                bool isStatic();

                /**
                 * Does the producer support history queries?
                 *
                 * @return Returns the history.
                 */
                bool isHistory();

                /**
                 * Does the producer support latest queries?
                 *
                 * @return Returns the latest.
                 */
                bool isLatest();

                /**
                 * Returns a String representation of this object.
                 */
                std::string toString();

                /**
                 * Returns the ignoreSlowConsumer flag.
                 *
                 * @return ignoreSlowConsumers.
                 *
                 * @deprecated
                 */
                bool ignoreSlowConsumers();

                /**
                 * Sets the secondaryProducer flag.
                 *
                 * @param secondaryProducer Is the producer a secondary producer?
                 *
                 * @deprecated @see #setSecondaryProducer(boolean)
                 */
                void setRepublisher(bool secondaryProducer);

                /**
                 * Sets the republisher flag.
                 *
                 * @param secondaryProducer Is the producer part of a republisher?
                 */
                void setSecondaryProducer(bool secondaryProducer);

            private:
                /** Is the producer part of a republisher? */
                bool m_secondaryProducer;

                /** Does the producer support continuous queries? */
                bool m_continuous;

                /** Does the producer support static queries? */
                bool m_static;

                /** Does the producer support history queries? */
                bool m_history;

                /** Does the producer support latest queries? */
                bool m_latest;

                /** Should the producer ignore slow consumers? @deprecated */
                bool m_ignoreSlowConsumers;
        };
    }
}
#endif
