/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef QueryProperties_H
#define QueryProperties_H

#include "rgma/TimeInterval.h"

namespace glite
{
    namespace rgma
    {
        /**
         * The type of a consumer query.  There are six types:
         *
         * <ul>
         * <li>
         * HISTORY: <code>QueryProperties.HISTORY</code>
         * </li>
         * <li>
         * HISTORY over a time interval: <code>QueryProperties.HISTORY(TimeInterval)</code>
         * </li>
         * <li>
         * LATEST: <code>QueryProperties.LATEST</code>
         * </li>
         * <li>
         * LATEST over a time interval: <code>QueryProperties.LATEST(TimeInterval)</code>
         * </li>
         * <li>
         * CONTINUOUS: <code>QueryProperties.CONTINUOUS</code>
         * </li>
         * <li>
         * CONTINUOUS over a time interval:
         * <code>QueryProperties.CONTINUOUS(TimeInterval)</code>
         * </li>
         * </ul>
         */
        class QueryProperties
        {
            private:
                const static int C = 1;
                const static int L = 2;
                const static int H = 4;
                const static int S = 8;
                TimeInterval m_timeInterval;
                int m_query;
                bool m_hasTimeInterval;
            public:
                /** Stream data continuously, starting with the next inserted tuple. */
                const static QueryProperties CONTINUOUS;

                /** Retrieve data from LATEST producers. */
                const static QueryProperties LATEST;

                /** Retrieve data from HISTORY producers. */
                const static QueryProperties HISTORY;

                /** Retrieve data from STATIC producers. */
                const static QueryProperties STATIC;

                virtual ~QueryProperties();

                QueryProperties( const QueryProperties& properties );
                QueryProperties& operator=(const QueryProperties& properties );

                /**
                 * Returns a CONTINUOUS QueryProperties object with a time interval.
                 *
                 * @param timeInterval Time interval for query.
                 *
                 * @return A QueryProperties object.
                 */
                const static QueryProperties getContinuous(TimeInterval timeInterval);
                /**
                 * Returns a HISTORY QueryProperties object with a time interval.
                 *
                 * @param timeInterval Time interval for query.
                 *
                 * @return A QueryProperties object.
                 */
                const static QueryProperties getHistory(TimeInterval timeInterval);

                /**
                 * Returns a LATEST QueryProperties object with a time interval.
                 *
                 * @param timeInterval Time interval for query.
                 *
                 * @return A QueryProperties object.
                 */
                const static QueryProperties getLatest(TimeInterval timeInterval);

                /**
                 * Returns a STATIC QueryProperties object with a time interval.
                 *
                 * @param timeInterval Time interval for query.
                 *
                 * @return A QueryProperties object.
                 */
                const static QueryProperties getStatic(TimeInterval timeInterval);

                /**
                 * Determines if this is a CONTINUOUS query.
                 *
                 * @return true if a CONTINUOUS query, otherwise false.
                 */
                bool isContinuous();

                /**
                 * Determines if this is a HISTORY query.
                 *
                 * @return true if a HISTORY query, otherwise false.
                 */
                bool isHistory();

                /**
                 * Determines if this is a LATEST query.
                 *
                 * @return true if a LATEST query, otherwise false.
                 */
                bool isLatest();

                /**
                 * Determines if this is a STATIC query.
                 *
                 * @return true if a STATIC query, otherwise false.
                 */
                bool isStatic();

                /**
                 * Returns the time interval associated with this QueryProperties.
                 *
                 * @return The time interval associated with this QueryProperties, or
                 *         <code>null</code> if no time interval has been set.
                 */
                TimeInterval getTimeInterval();

                /**
                 * Compares two QueryPropertiess.
                 *
                 * @param qt Another QueryProperties.
                 *
                 * @return True if the QueryPropertiess are equal.
                 */
                bool equals(QueryProperties qt);

                /**
                 * Determines if this QueryProperties has an associated TimeInterval.
                 *
                 * @return true if it has a TimeInterval, otherwise false.
                 */
                bool hasTimeInterval();

            protected:
                /**
                 * Constructs a QueryProperties object.
                 *
                 * @param query The type of query: C, L or H
                 */
                QueryProperties(int query);

                /**
                 * Creates a new QueryProperties object of type <code>query</code> with a time
                 * interval.
                 *
                 * @param query Query type.
                 * @param timeInterval The period over which to retrieve data.
                 */
                QueryProperties(int query, TimeInterval timeInterval);
                /**
                 * Gets the query type.
                 *
                 * @return The query type: CONTINUOUS, LATEST, HISTORY or CONTINOUS_OLD
                 */
                int getQuery();

        };
    }
}
#endif                                            // QueryProperties_H
