/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
 #ifndef REGISTRY_H
#define REGISTRY_H

#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RGMAException.h"
#include "rgma/ResourceImpl.h"
#include "rgma/ProducerTableEntryList.h"

namespace glite
{
    namespace rgma
    {
        /**
         * Admin API for the Registry.
         */
        class Registry: public ResourceImpl
        {
            public:
                virtual ~Registry(){}
                /**
                 * Removes the Registry instance for the given VO. Request is
                 * forwarded to <code>registryService</code> for processing.
                 *
                 * @param vdbName The VDB that owns the Registry instance.
                 * @param registryService Registry service URL.
                 *
                 * @throws RemoteException If the service can not be contacted.
                 * @throws RGMAException If the Registry instance can not be removed or does
                 *         not exist.
                 */
                static void destroyRegistry( std::string vdbName, std::string registryService)
                    throw(RGMAException, RemoteException);
                /**
                 * Creates a Registry instance for the given VDB. Request is
                 * forwarded to <code>registryService</code> for processing.
                 *
                 * @param vdbName The VDB that owns the Registry instance.
                 * @param registryService Registry service URL.
                 *
                 * @throws RemoteException If the service can not be contacted.
                 * @throws RGMAException
                 */
                static void createRegistry( std::string vdbName, std::string registryService)
                    throw(RGMAException, RemoteException);

                /**
                 * Returns a list of all producers, with their details, for use by the
                 * Browser (or other admin) applications.

                 * @param vdbName Name of VDB from which to find producers.
                 * @param tableName Name of table
                 *
                 * @return A list of ProducerTableEntry objects, one for each producer.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If an error occurs while carrying out the request
                 *         by the Registry.
                 */
                static ProducerTableEntryList getAllProducersForTable(std::string vdbName, std::string tableName)
                    throw( RGMAException, RemoteException );
                /**
                 * Gets the version of this implementation.
                 *
                 * @return The version number.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 */
                static std::string getVersion()throw(RemoteException);

                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param parameter Parameter to property
                 *
                 * @return Current value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name, std::string parameter)
                    throw(RemoteException, RGMAException);
                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 *
                 * @return Current value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name)
                    throw(RemoteException, RGMAException);
                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 *  @param parameter Parameter to property
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value, std::string parameter)
                    throw(RemoteException, RGMAException);

                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value)
                    throw(RemoteException, RGMAException);
        };
    }
}
#endif
