#ifndef RESOURCE_IMPL_H
#define RESOURCE_IMPL_H
#include "rgma/ResourceEndpoint.h"
#include "rgma/Resource.h"
#include "rgma/TimeInterval.h"
#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RGMAException.h"
#include "rgma/ServletConnection.h"
#include "log4cxx/logger.h"
#include "log4cxx/level.h"
#include "log4cxx/helpers/loglog.h"
#include "log4cxx/propertyconfigurator.h"
#include "log4cxx/helpers/exception.h"
#include <string>

namespace glite
{
    namespace rgma
    {
        /**
         * An object managed by a Web Service that an API can interact with. Either a
         * Producer or a Consumer.
         */
        class ResourceImpl :public Resource
        {
            protected:
                ResourceEndpoint m_endPoint;

                /**
                 * Will create a servlet connection if one does not exist or return a
                 * pointer to one that has been created previously
                 *
                 */
                ServletConnection* getServletConnection() throw(RGMAException);
            private:
                ServletConnection* m_servletConnection;
                log4cxx::LoggerPtr m_logger;

            public:

                virtual ~ResourceImpl();
                ResourceImpl();
                /**
                 * Gets the endpoint of the service/resource the API communicates with.
                 *
                 * @return The service/instance identifier
                 */
                virtual ResourceEndpoint getEndpoint();

                /**
                 * Retrieves the resource's termination interval.
                 *
                 * @return a pointer to the termination interval as a TimeInterval.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found
                 *
                 * @see #setTerminationInterval
                 */
                virtual TimeInterval getTerminationInterval()
                    throw(RemoteException, UnknownResourceException);
                /**
                 * Sets the termination interval for this resource.
                 *
                 * @deprecated
                 *
                 * @param terminationInterval The time interval after which the resource
                 *        may destroy itself if no contact has been made from the API.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found
                 * @throws RGMAException If the termination interval is invalid (e.g. in
                 *         the past)
                 *
                 *
                 * @see #getTerminationInterval
                 */
                virtual void setTerminationInterval(TimeInterval &time_interval)
                    throw(RemoteException, UnknownResourceException, RGMAException);
                /**
                 * Closes and destroys the resource.  The resource will no longer be
                 * available to any component.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found
                 * @deprecated
                 */
                virtual void destroy()
                    throw(RemoteException, UnknownResourceException);
                /**
                 * Closes the resource.  The resource will no longer be available through
                 * this API and will be destroyed once it has finished interacting with
                 * other components.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found
                 */
                virtual void close()
                    throw(RemoteException, UnknownResourceException);
                /**
                 * Indicates to the resource that this API is still alive.  This should be
                 * used to keep a resource alive when no other method needs to be called.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws UnknownResourceException If the resource could not be found
                 */
                virtual void showSignOfLife()
                    throw(RemoteException, UnknownResourceException);

        };
    }
}
#endif                                            // RESOURCE_H
