/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
 #ifndef SCHEMA_H
#define SCHEMA_H

#include "rgma/RGMAException.h"
#include "rgma/RemoteException.h"
#include "rgma/StringList.h"
#include "rgma/TableAuthorization.h"
#include "rgma/TableDefinition.h"
#include "rgma/IndexList.h"
#include <string>
namespace glite
{
    namespace rgma
    {
        /**
         * Admin API for the Schema.
         */
        class Schema
        {
            public:

                /**
                 * Creates a Schema instance for the given VDB.  Request is
                 * forwarded to <code>schemaService</code> for processing.
                 *
                 * @param vdbName The VDB that owns the Schema instance.
                 * @param schemaService Schema service URL.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the Schema cannot be created.
                 */
                static void createSchema(std::string vdbName, std::string schemaService)
                    throw( RGMAException, RemoteException);

                /**
                 * Removes the schema instance for the given VO.  Request is
                 * forwarded to <code>schemaService</code> for processing.
                 *
                 * @param vdbName The VDB that owns the Schema instance.
                 * @param schemaService Schema service URL.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the Schema instance cannot be removed or does
                 *         not exist.
                 */
                static void destroySchema(std::string vdbName, std::string schemaService)
                    throw( RGMAException, RemoteException);

                /**
                 * Gets a list of all tables in the schema for the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 *
                 * @return A StringList of table names.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 */
                static StringList getAllTables(std::string vdbName)
                    throw( RGMAException, RemoteException );
                /**
                 * Gets the definition for table <code>tableName</code> in the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 * @param tableName Name of table.
                 *
                 * @return Table definition.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the table could not be found
                 */
                static TableDefinition getTableDefinition(std::string vdbName, std::string tableName)
                    throw( RGMAException, RemoteException);

                /**
                 * Gets the list of indexes defined on the given table in the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 * @param tableName Name of table.
                 *
                 * @return List of Index objects.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the table could not be found
                 */
                static IndexList getTableIndexes(std::string vdbName, std::string tableName)
                    throw( RGMAException, RemoteException);

                /**
                 * Sets the authorization rules for the given table.
                 *
                 * @param vdbName Name of VDB.
                 * @param tableName Table name.
                 * @param tableAuthz Table authorization details (for no authorization, set
                 *        as <code>null</code>).
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the table does not exist or the rules are invalid.
                 */
                static void setAuthorizationRules(std::string vdbName, std::string tableName,
                    TableAuthorization tableAuthz) throw( RGMAException, RemoteException);

                /**
                 * Gets the authorization rules for the given table.
                 *
                 * @param vdbName Name of VDB.
                 * @param tableName Table name.
                 *
                 * @return Table authorization details.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the table does not exist.
                 */
                static TableAuthorization getAuthorizationRules(std::string vdbName, std::string tableName)
                    throw( RGMAException, RemoteException);

                /**
                 * Creates a table in the Schema instance for the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 * @param createTableStatement SQL CREATE TABLE statement.
                 * @param tableAuthz Table authorization details (for no authorization, set
                 *        as <code>null</code>)
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the table already exists and the Schema
                 * definition given by createTableStatement does not match with the
                 * stored R-GMA Schema description.
                 */
                static void createTable(std::string vdbName, std::string createTableStatement,
                    TableAuthorization tableAuthz) throw( RGMAException, RemoteException);

                /**
                 * Drops a table from the Schema instance for the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 * @param tableName Table to drop.
                 *
                 * @throws RemoteException If the service can not be contacted.
                 * @throws RGMAException If the table does not exist.
                 */
                static void dropTable(std::string vdbName, std::string tableName)
                    throw( RGMAException, RemoteException);

                /**
                 * Creates an index in the Schema instance for the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 * @param createIndexStatement SQL CREATE INDEX statement.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the index already exists or the CREATE INDEX statement
                 * is invalid.
                 */
                static void createIndex(std::string vdbName, std::string createIndexStatement)
                    throw( RGMAException, RemoteException);

                /**
                 * Drops an index from the Schema instance for the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 * @param indexName Index to drop.
                 *
                 * @throws RemoteException If the service can not be contacted.
                 * @throws RGMAException If the index does not exist.
                 */
                static void dropIndex(std::string vdbName, std::string indexName)
                    throw( RGMAException, RemoteException);

                /**
                 * Creates an view in the Schema instance for the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 * @param createViewStatement SQL CREATE VIEW statement.
                 * @param viewAuthz View authorization details (for no authorization, set
                 *        as <code>null</code>)
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the view already exists or the CREATE VIEW statement
                 * is invalid.
                 */
                static void createView(std::string vdbName, std::string createViewStatement,
                    TableAuthorization viewAuthz) throw( RGMAException, RemoteException);

                /**
                 * Drops an view from the Schema instance for the given VDB.
                 *
                 * @param vdbName Name of VDB.
                 * @param viewName View to drop.
                 *
                 * @throws RemoteException If the service can not be contacted.
                 * @throws RGMAException If the view does not exist.
                 */
                static void dropView(std::string vdbName, std::string viewName)
                    throw( RGMAException, RemoteException);

                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param parameter Parameter to property
                 *
                 * @return Current value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name, std::string parameter)
                    throw(RemoteException, RGMAException);
                /**
                 * Gets the value of the specified service property.
                 *
                 * @param name Name of property.
                 *
                 * @return Current value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static std::string getProperty(std::string name)
                    throw(RemoteException, RGMAException);
                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 *  @param parameter Parameter to property
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value, std::string parameter)
                    throw(RemoteException, RGMAException);

                /**
                 * Changes the value of the specified service property.
                 *
                 * @param name Name of property.
                 * @param value New value of property.
                 *
                 * @throws RemoteException If the service could not be contacted.
                 * @throws RGMAException If the name/parameter combination is invalid.
                 */
                static void setProperty(std::string name, std::string value)
                    throw(RemoteException, RGMAException);

                static std::string getVersion()throw(RemoteException);
        };
    }
}
#endif
