/* Copyright (c) 2001-2003 EU DataGrid. */
/* For license conditions see http://www.eu-datagrid.org/license.html */

#ifndef INFO_SERVLETCONNECTION_H
#define INFO_SERVLETCONNECTION_H
#define OPENSSL_NO_KRB5
#include "rgma/RGMAException.h"
#include "rgma/RemoteException.h"
#include "rgma/SSLContext.h"
#include "log4cxx/logger.h"
#include "log4cxx/level.h"
#include "log4cxx/helpers/loglog.h"
#include "log4cxx/propertyconfigurator.h"
#include "log4cxx/helpers/exception.h"
#include "rgma/Socket.h"
#include <openssl/ssl.h>
#include <string>

namespace glite
{
    namespace rgma
    {

        /**
         * Servlet interaction
         */
        class ServletConnection
        {
            // This little class chops up URLs

            public:

                explicit ServletConnection(std::string servletURL) throw (RGMAException);

                ServletConnection(std::string servletURL, int connectionId) throw (RGMAException);

                virtual ~ServletConnection();

                /**
                 * add a string parameter (name value pair).
                 * @param name name of parameter
                 * @param value value of parameter
                 */
                void addParameter(std::string name, std::string value) throw (RGMAException);

                /**
                 * add a char* parameter (name value pair).
                 * @param name name of parameter
                 * @param value value of parameter
                 */
                void addParameter(std::string name, const char* value) throw (RGMAException);

                /**
                 * add an integer parameter (name value pair).
                 * @param name name of parameter
                 * @param value value of parameter
                 */
                void addParameter(std::string name, int value) throw (RGMAException);

                /**
                 * add a double parameter (name value pair).
                 * @param name name of parameter
                 * @param value value of parameter
                 */
                void addParameter(std::string name, double value) throw (RGMAException);

                /**
                 * add a boolean parameter (name value pair).
                 * @param name name of parameter @param value value of parameter
                 */
                void addParameter(std::string name, bool value) throw (RGMAException);

                /**
                 * add a long parameter (name value pair).
                 * @param name name of parameter
                 * @param value value of parameter
                 */
                void addParameter(std::string name, long value) throw (RGMAException);

                /**
                 * Connect with specified operation.  A proxy cache may be used for all accesses
                 * except to localhost, by settin the environment variable http_proxy to it.
                 * e.g. export http_proxy=wwwcache.rl.ac.uk:8080
                 *
                 * @param operation name of operation the servlet is to perform.
                 * The operation name is appended to the base servlet URI to which are
                 * appended any parameters to form the complete URI
                 * @param response returned XML string
                 */
                void connect(std::string operation,
                    std::string& response) throw (RGMAException, RemoteException);

                int getConnectionId() const;

                std::string getServletURL() const;

                std::string toString() const;

            protected:
                /**
                 * For testing - return the current parameters.
                 *
                 * @return parameters attribute
                 */
                std::string getParameters() const { return m_parameters; }

            private:
                /**
                 * Throw an exception if servletURL does not have a sensible format
                 */
                void checkURL() const throw (RGMAException);

                /**
                 * Make HTTP or HTTPS request and obtain response.
                 * Used by connect() method.
                 * @param url HTTP or HTTPS request
                 * @param response reference to string to store the response in
                 */

                void httpRequest(const std::string url,
                    std::string& response) throw (RGMAException, RemoteException);

                /*
                 * Return the HTML document with the HTTP header stripped off, plus any chunking
                 * removed.  It assumes that there is no HTTP trailer.
                 *
                 * @param document HTML document including HTTP header
                 * @return HTML document with header stripped off
                 */
                std::string stripHeader(std::string document);

                //static const class APILogging& cat;
                int m_connectionId;
                std::string m_parameters;
                std::string m_servletURL;
                Socket* m_sock;
                bool m_firstConnect;
                log4cxx::LoggerPtr m_logger;

        };

    }                                             // info
}                                                 // edg
#endif                                            // EDG_INFO_SERVLETCONNECTION_H
