/*
 *  Copyright (c) 2004 on behalf of the EU EGEE Project:
 *  The European Organization for Nuclear Research (CERN),
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Datamat Spa, Italy
 *  Centre National de la Recherche Scientifique (CNRS), France
 *  CS Systeme d'Information (CSSI), France
 *  Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 *  Universiteit van Amsterdam (UvA), Netherlands
 *  University of Helsinki (UH.HIP), Finland
 *  University of Bergen (UiB), Norway
 *  Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 */
#ifndef STORAGELOCATION_H
#define STORAGELOCATION_H

#include <string>

namespace glite
{
    namespace rgma
    {
        /**
         * The location where tuples will be stored.  There are three options:
         *
         * <ul>
         * <li>
         * MEMORY: <code>Storage::MEMORY</code>
         * </li>
         * <li>
         * default DATABASE: <code>Storage::DATABASE</code>
         * </li>
         * <li>
         * specific DATABASE: <code>Storage::DATABASE(location, user,
         * password)</code>
         * </li>
         * <li>
         * FILE: <code>Storage.FILE</code>
         * </li>
         * </ul>
         */
        class Storage
        {

            private:
                std::string m_location;
                std::string m_userName;
                std::string m_password;

                std::string m_logicalName;

                int m_type;

                bool compare(std::string a, std::string b);

            public:
                /** The default MEMORY storage location. */
                const static Storage MEMORY;

                /** The default DATABASE storage location. */
                const static Storage DATABASE;

                /** The default FILE storage location. */
                const static Storage FILE;

                virtual ~Storage();

                /**
                 * copy constructor
                 */

                Storage(const Storage& storage);

                /**
                 * assignment operator
                 */

                Storage& operator=(const Storage& storage);

                /**
                 * Gets a DATABASE StorageLocation object for the specified database.
                 *
                 * @param url The URL of the storage system (e.g. dbms)
                 * @param userName The user name for the storage system.
                 * @param password The password for the storage system.
                 * @deprecated
                 * @return A StorageLocation object for the specified database.
                 */
                static Storage getDatabase(const std::string location,
                    const std::string userName, const std::string password);

                /**
                 * Gets a DATABASE Storage object for the specified database.
                 *
                 * @param logicalName The logical name of the storage system
                 *
                 * @return A Storage object for the specified database.
                 */
                static Storage getDatabase(const std::string logicalName);

                /**
                 * Determines if this storage system is a database.
                 *
                 * @return True if the storage system is a database.
                 */
                bool isDatabase();

                /**
                 * Determines if this storage system is a file.
                 * @deprecated
                 * @return True if the storage system is a file.
                 */
                bool isFile();

                /**
                 * Determines if this storage system is memory.
                 *
                 * @return True if the storage system is memory.
                 */
                bool isMemory();

                /**
                 * Gets the password for this storage system.
                 *
                 * @return Password as a String
                 */
                std::string getPassword();

                /**
                 * Gets the location for this storage system.
                 *
                 * @return location as a String
                 */
                std::string getLocation();

                /**
                 * Gets the user name for this storage system.
                 *
                 * @return User name as a String
                 */
                std::string getUserName();

                /**
                 * Gets the user name for this storage system.
                 *
                 * @return User name as a String
                 */
                std::string getLogicalName();

                /**
                 * Compares this Storage to the specified location.
                 *
                 * @param storageLocation Another Storage object.
                 *
                 * @return true if the specified location is identical to this location.
                 */
                //bool equals(Storage storage) ;

                /**
                 * Determines if this Storage contains storage details.
                 *
                 * @return true if it has storage details, otherwise false.
                 */
                bool hasDetails();

                /**
                 * Returns a hash code value for this Storage object.
                 *
                 * @deprecated
                 *
                 * @return Hash code value for this Storage object.
                 */
                int hashCode();

                /**
                 * Compares this StorageLocation to the specified location.
                 *
                 * @param storage Another Storage object.
                 *
                 * @return true if the specified storage location is identical
                 * to this one false otherwise.
                 */
                bool operator==(const Storage storage);

                /**
                 * Compares this StorageLocation to the specified location.
                 *
                 * @param storage Another StorageLocation object.
                 *
                 * @return false if the specified storage location is identical
                 * to this one false otherwise.
                 */
                bool operator!=(const Storage storage);

                /**
                 * Compares this StorageLocation to the specified location.
                 *
                 * @param storage Another Storage object.
                 *
                 * @return true if the specified storage location is identical
                 * to this one false otherwise.
                 */
                bool equals(Storage Storage);

                Storage(){};
                int getType();

            protected:
                /** The memory storage location. */
                const static int M = 1;
                /** The database storage location. */
                const static int D = 2;
                /** The file storage location. */
                const static int F = 4;

                /**
                 * Creates a new Storage object.
                 *
                 * @param location Storage.MEMORY or Storage.DATABASE
                 */
                Storage(int type);

                /**
                 * Creates a new Storage object.
                 *
                 * @param type Type of storage.
                 * @param logicalName Logical name of storage medium.
                 */
                Storage( const int type, const std::string logicalName );

                /**
                 * Creates a new DATABASE StorageLocation object.
                 *
                 *
                 * @param location The URL of the storage system (e.g. dbms)
                 * @param userName userName to access storage medium.
                 * @param password password to access storage medium.
                 */
                Storage( const std::string location, const std::string userName,
                    const std::string password);

        };
    }
}
#endif                                            // STORAGELOCATION_H
