/* Copyright (c) 2001-2003 EU DataGrid. */
/* For license conditions see http://www.eu-datagrid.org/license.html */

#ifndef RGMA_XMLCONVERTER_H
#define RGMA_XMLCONVERTER_H

#include "rgma/RGMAException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/ResultSet.h"
#include "rgma/RGMAWarning.h"
#include "log4cxx/logger.h"
#include "log4cxx/level.h"
#include "log4cxx/helpers/loglog.h"
#include "log4cxx/propertyconfigurator.h"
#include "log4cxx/helpers/exception.h"
#include <stack>
#include <string>
#include <xercesc/dom/deprecated/DOM.hpp>
#include <xercesc/sax/ErrorHandler.hpp>

using namespace XERCES_CPP_NAMESPACE;

namespace glite
{
    namespace rgma
    {
        /**
         * Parses XML and constructs ResultSet
         */
        class XMLConverter : public ErrorHandler
        {

            public:
                /**
                 *  Helper class to store instance fields of RGMAExceptions 
                 * */
                class Helper
                {
                    public:
                        std::string message;
                        bool isRecoverable;
                        std::string source;
                        std::string type;

                        Helper(std::string type, std::string message,
                            bool isRecoverable, std::string source);
                        Helper(const Helper& other);
                };

                /**
                 * Constructor
                 */
                XMLConverter() throw (RGMAException);

                /**
                 * Destructor
                 */
                virtual ~XMLConverter();

                /**
                 * Generate result set from XML string. This generates an empty set if the
                 * input string is empty
                 *
                 * @param resultSet a reference to the result set to be modified
                 * @param xml the XML document to be parsed to generate the result set
                 */
                void convertXMLResponse(ResultSet& resultSet,
                    const std::string& xml) throw (RGMAException, UnknownResourceException);

                /**
                 * Warning handler for the SAX ErrorHandler interface
                 */
                void warning(const SAXParseException& exception);

                /**
                 * Error handler for the SAX ErrorHandler interface
                 */
                void error(const SAXParseException& exception);

                /**
                 * Fatal error handler for the SAX ErrorHandler interface
                 */
                void fatalError(const SAXParseException& exception);

                void resetErrors();

            private:

                RGMAException makeRGMAException(const char* severity,
                    const char* operation,
                    const char *message);

                RGMAException makeRGMAException(const char* severity,
                    const char* operation,
                    const SAXParseException& e);

                RGMAException makeRGMAException(const char* severity,
                    const char* operation,
                    const XMLException& e);

                // This has the added methods for setting the meta data and rows
                // or the result set
                class ResultSetImpl:public ResultSet
                {
                    public:
                        ResultSetImpl();
                        void setMetaData(ResultSetMetaData& metaData);
                        void setEndOfResults( bool endOfResults);
                        void addRow(Tuple tuple);
                        void setWarning(RGMAWarning warning);
                };

                /**
                 * If a proxy cache is being used and it fails to retrieve the required
                 * document from the servlet then it will respond with an HTML document
                 * describing the error.
                 * This method checks if the document is an HTML one and then attempts
                 * to extract the title for appending to the error message which it
                 * puts in the RGMAException which it throws.  This will often contain
                 * the error that occurred at the proxy cache.
                 *
                 * @param document the XML or HTML document to check
                 */
                void checkForHtml(const std::string& document) const throw (RGMAException);

                DOM_Node skipTextNode( DOM_Element& element );

                /**
                 * Method to parse using XercesDOMParser
                 * @param document the XML document to parse
                 * @return parsed Document
                 */
                DOM_Document DOMParse(const std::string& document) throw (RGMAException);

                std::string  schemaLocation;

                log4cxx::LoggerPtr m_logger;
        };
    }
}
#endif                                            //RGMA_XMLCONVERTER_H
